/*
f-cpu/qdcpoc/qdcpoc.h
"Quick and dirty coded proof of concept" for the F-CP0 pipeline.
created Wed Jul 11 01:36:14 2001
(C) Whygee@f-cpu all rights reserved (you're warned, it's shit !)

this file is included by f-cpu/qdcpoc/main.c

*/

/* the usual libraries */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>  /* for fstat() */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* F-CPU compatibility interface : */
#include "../opcodes.h"

/* the F-CPU standard types : */
typedef unsigned char u8;
typedef   signed char s8;
typedef unsigned short int u16;
typedef   signed short int s16;
typedef unsigned long int u32;
typedef   signed long int s32;
typedef unsigned long long int u64;
typedef   signed long long int s64;

typedef u64 UMAX;   /* these types must be updated */
typedef s64 SMAX;   /* if the CPU can handle wider data */

/* depth of the scheduler's FIFO */
#define DEPTH_QUEUE 5

/****************************************************
      FC0 state variables and pipeline latches :
 ****************************************************/


/********************
  The Register set
  (or R7 for short)
 ********************/
UMAX reg_set[64]; /* beware : assume reg_set[0]==0 */

/* interface for writing in the R7 : */
UMAX R7write_mask0,    /* The write_mask is a bit field */
     R7write_port0,    /* where write_enables are 1s */
     R7write_mask1,
     R7write_port1;
int R7write_address0,
    R7write_address1; /* 0 if no register to write, 1-63 otherwise */

/* interface for reading in the R7 : */
UMAX R7read_port0,
     R7read_port1,
     R7read_port2;
/* int R7read_address0,    The variables are now local in fc0_cycle_decode().
       R7read_address1,
       R7read_address2;    0 if no register to write, 1-63 otherwise */

/* R7 flags : */
u8 null_flags[64]; /* each char is 0 if the corresponding reg. is 0 */
u64 NULLflag, /* the combination of all the partial 0 flags */
    LSBflag, /* here we don't need partial results, */
    MSBflag; /* all 63 bits are exploited */

UMAX mask_LUT[] = {
  0x00000000000000FFLL, /*  8-bit write mask */
  0x000000000000FFFFLL, /* 16-bit write mask */
  0x00000000FFFFFFFFLL, /* 32-bit write mask */
  0xFFFFFFFFFFFFFFFFLL, /* 64-bit write mask (SIMD) */
  0x000000000000FFFFLL, /* LOADCONS */    /* degenerate */
  0x00000000FFFF0000LL,
  0x0000FFFF00000000LL,
  0xFFFF000000000000LL,
  0xFFFFFFFFFFFFFFFFLL, /* LOADCONSX */   /* degenerate */
  0xFFFFFFFFFFFF0000LL,
  0xFFFFFFFF00000000LL,
  0xFFFF000000000000LL                    /* degenerate */
};

/********************
   The "Xbar" :
 ********************/

/* int Xbar_write_address0, Xbar_write_address_x are used to determine from */
/*     Xbar_write_address1, which port to read the future contents of the R7 write buses */
/* update : Sat Jul 21 03:46:55 2001 : turned into local variables in fc0_cycle_Xbar_write() */

int Xbar_read_address0,
    Xbar_read_address1,
    Xbar_read_address2; /* if=0 => do nothing */
UMAX Xbar_read_port0,
     Xbar_read_port1, /* these ports feed all the EUs */
     Xbar_read_port2;
int Xbar_EU_ROP2,       /* delays the signal */
    Xbar_ROP2_mode;

/********************
    the ROP2 EU :
 ********************/
UMAX Xbar_ROP2_port; /* output port */

/* extract from the F-CPU manual : the ROP2 lookup table
   opcode   real code   Function  Symbolic name
     000      0001       A AND B      AND
     001      0010       A AND /B     ANDN
     010      0110       A XOR B      XOR
     011      0111       A OR B       OR
     100      1000       A NOR B      NOR
     101      1001       A XNOR B     XNOR
     110      1101       A OR /B      ORN
     111      1110       A NAND B     NAND
*/

unsigned int ROP2_function_LUT[8] = {
  1, 2, 6, 7, 8, 9, 0xD, 0xE
};

UMAX ROP2_function_bit0,
     ROP2_function_bit1,
     ROP2_function_bit2,
     ROP2_function_bit3;


/* Output of the other EUs : */
UMAX Xbar_ASU_port0,
     Xbar_ASU_port1;


/********************
  the decoder :
 ********************/

/* outputs to the Xbar/EUs : */
UMAX DEC_imm_port,  /* immediate data */
     DEC_NIP,  /* Next Instruction Pointer (CIP+4) */
     DEC_CIP;  /* Current Instruction Pointer */

int EU_ROP2,       /* enable signal (triggers the computation) */
    ROP2_mode,     /* 0: normal, 1: AND, 2: OR, 3: select  */
    ROP2_function, /* 0-7, + translated & buffered during Xbar read cycle */
    emit_instruction,  /* set when the instruction can be issued. */
    valid_opcode,
    pointer_ready,     /* currently unused */
    condition_true,    /* warning : doubtful polarity. */
    register0_ready,   /* set to 1 if the corresponding register */
    register1_ready,   /* is ready (not used in the FIFO) */
    register2_ready,
    RW2_ready,
    register0_on_Xbar0,   /* bypass condition : */
    register1_on_Xbar0,   /*  */
    register2_on_Xbar0,
    RW2_on_Xbar0,
    register0_on_Xbar1,
    register1_on_Xbar1,
    register2_on_Xbar1,
    RW2_on_Xbar1;

/* internal : */
u8 current_opcode,
   RW2,
   imm8;
u16 imm16;
int condition_code; /* field from the opcode. Used during
 issue to determine if the condition is a valid/legal combination. */

/* Communication with the Fetcher : */
u32 instruction_buffer; /* the currently decoded instruction */
int get_next_instruction; /* signal that asks the fetcher to do its job... */

/********************
   Fetcher :
 ********************/

UMAX FETCH_NIP,  /* Next Instruction Pointer (CIP+4) */
     FETCH_CIP;  /* Current Instruction Pointer */

u32 fetcher_buffer_size; /* number of instructions in the buffer (size in bytes/4) */
u32 *fetcher_buffer=NULL;

/* much more to come !!! */

/********************
    Scheduler :
 ********************/

typedef struct s_queue_entry {
  int write0 : 6;   /* register number */
  int unit0 :  3;   /* from which unit to get the data */
  int mask0 :  8;   /* write mask (compressed) */
  int busy0 :  1;   /* this is set when the write slot is not empty */
  int write1 : 6;
  int unit1 :  3;
  int mask1  : 8;
  int busy1 :  1;
} t_queue_entry;

t_queue_entry sched_queue[DEPTH_QUEUE];

typedef struct {

  /* latency :
  indicates which queue entry will be filled */
  int latency_zero : 1;      /* nop (could be removed ?) */
  int latency_direct : 1;    /* move, loadcons */
  int latency_cycle_1 : 1;   /* rop2, inc */
  int latency_cycle_2 : 1;   /* ASU */
  int latency_multiply : 1;  /* imul (more ports ?) */
  int latency_idiv : 1;      /* idiv */

  /* opcode format :
  indicates which fields are necessary before we issue the instruction */
  int need_src0 : 1;
  int need_src1 : 1;
  int need_src2 : 1;
  int need_rw2  : 1;
  int need_cond : 1;

  /* queue reservation : */
  int need_w1 : 1; /* indicate that we want 1 write slot
  (in this case, src2 is written to either slot0 or slot1) */
  int need_w2 : 1; /* indicate that we want 2 simultaneous write slots
  (in this case, src2 and rw2 are written to the 2 slots) */

  /* use of the constants :
   it directly drives the Xbar */
  int op_imm8 : 1;
  int op_imm16 : 1;

  /* more will be added as special cases are discovered
  int  : 1;
  int  : 1;
  int  : 1;
  int  : 1;
  int  : 1;
  int  : 1;
 */

} opcode_lookup_type;

opcode_lookup_type opcode_LUT[256], opcode_type;



/********************
   IDIV :
 ********************/

int IDIV_complete, IDIV_counter;
u8 IDIV_write_destination, IDIV_write_destination_bis,
   IDIV_write_mask0, IDIV_write_mask1,
   IDIV_unit0, IDIV_unit1;

